package com.muyu.marketing.team.controller;


import com.dtflys.forest.annotation.Put;
import com.muyu.common.core.domain.Result;
import com.muyu.common.core.web.page.TableDataInfo;
import com.muyu.marketing.domain.model.*;
import com.muyu.marketing.domain.req.ActivityTeamInfoSaveReq;
import com.muyu.marketing.domain.req.ActivityTeamInfoUpdReq;
import com.muyu.marketing.domain.req.ProjectSkuInfoAddReq;
import com.muyu.marketing.domain.req.TeamInfoListReq;
import com.muyu.marketing.domain.resp.ProjectFindSkuInfoResp;
import com.muyu.marketing.domain.resp.TeamInfoFindByIdResp;
import com.muyu.marketing.domain.resp.TeamInfoListResp;
import com.muyu.marketing.team.service.ActivityTeamInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.util.List;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 营销团购活动控制层
 *
 * @author DongZeLiang
 * @date 2024-11-20 14:25
 */
@RestController
@RequestMapping("/team")
public class ActivityTeamController {

    @Autowired
    private ActivityTeamInfoService activityTeamInfoService;;

    /**
     * 查询营销团购活动列表
     * @param teamInfoListReq 活动查询入参
     * @return 活动响应结果
     */
    @PostMapping("/list")
    public Result<TableDataInfo<TeamInfoListResp>> list(@RequestBody TeamInfoListReq teamInfoListReq) {
        TableDataInfo<ActivityTeamInfoListModel> tableDataInfo = activityTeamInfoService.query(teamInfoListReq.buildQueryModel());
        List<TeamInfoListResp> respList = tableDataInfo.getRows().stream().map(TeamInfoListResp::listModelBuild).toList();
        return Result.success(
                new TableDataInfo<>(){{
                    setRows(respList);
                    setTotal(tableDataInfo.getTotal());
                }}
        );
    }
    @PostMapping("/saveTeam")
    public boolean saveTeam (@RequestBody ActivityTeamInfoSaveReq activityTeamInfoSaveReq){
        List<ProjectSkuInfoAddReq> projectSkuInfoAddReqList = activityTeamInfoSaveReq.getProjectSkuInfoAddReqList();

        return activityTeamInfoService.saveActivityTeamInfo(
                ActivityTeamInfoAddModel.activityTeamInfoAddModelBuild(activityTeamInfoSaveReq, (activityTeamInfoAddModelBuilder) -> {
                        List<ActivityTeamProductSkuAddModel> collect = projectSkuInfoAddReqList.stream().map(projectSkuInfoAddReq -> {
                            return ActivityTeamProductSkuAddModel.builder()
                                    .teamStock(projectSkuInfoAddReq.getTeamStock())
                                    .teamPrice(projectSkuInfoAddReq.getTeamPrice())
                                    .sku(projectSkuInfoAddReq.getSku())
                                    .productId(activityTeamInfoSaveReq.getProductId())
                                    .build();
                        }).collect(Collectors.toList());
                        return activityTeamInfoAddModelBuilder
                                .activityTeamProductSkuAddModelList(collect)
                                .build();
                })
        );
    }
    @PostMapping("/findById/{id}")
    public Result<TeamInfoFindByIdResp> findTeamById(@PathVariable Long id){
        return Result.success(TeamInfoFindByIdResp.teamInfoFindByIdBuild(activityTeamInfoService.findTeamById(id)));
    }
    @PutMapping("/updateByTeamId")
    public Result updateByTeamId(@RequestBody ActivityTeamInfoUpdReq activityTeamInfoUpdReq){
        activityTeamInfoService.updateByTeamId(ActivityTeamInfoUpdReqModel.activityTeamInfoUpdReqModelBuild(activityTeamInfoUpdReq));
        return Result.success();
    }
}
