package com.muyu.marketing.team.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.muyu.common.core.utils.StringUtils;
import com.muyu.common.core.web.page.TableDataInfo;
import com.muyu.marketing.domain.ActivityTeamInfo;
import com.muyu.marketing.domain.model.*;
import com.muyu.marketing.team.mapper.ActivityTeamInfoMapper;
import com.muyu.marketing.team.service.ActivityTeamInfoService;
import com.muyu.marketing.team.service.ActivityTeamOpenInfoService;
import com.muyu.marketing.team.service.ActivityTeamProductSkuInfoService;
import io.jsonwebtoken.lang.Assert;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;


@Service
public class ActivityTeamInfoServiceImpl extends ServiceImpl<ActivityTeamInfoMapper, ActivityTeamInfo> implements ActivityTeamInfoService {

    @Autowired
    private ActivityTeamOpenInfoService activityTeamOpenInfoService;

    @Autowired
    private ActivityTeamProductSkuInfoService activityTeamProductSkuInfoService;

    @Override
    public TableDataInfo<ActivityTeamInfoListModel> query(ActivityTeamInfoListQueryModel activityTeamInfoListQueryModel) {

        LambdaQueryWrapper<ActivityTeamInfo> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.like(StringUtils.isNotEmpty(activityTeamInfoListQueryModel.getKeyWord()), ActivityTeamInfo::getName, activityTeamInfoListQueryModel.getKeyWord());
        queryWrapper.like(StringUtils.isNotEmpty(activityTeamInfoListQueryModel.getStatus()), ActivityTeamInfo::getStatus, activityTeamInfoListQueryModel.getStatus());

        /**
         * Object<T> -> 创建对象的时候进行的占用
         * <T> Result<T> 以方法返回值为占用
         */
        Page<ActivityTeamInfo> activityTeamInfoPage = this.page(activityTeamInfoListQueryModel.buildPage(), queryWrapper);
        List<ActivityTeamInfo> activityTeamInfoList = activityTeamInfoPage.getRecords();
        List<ActivityTeamInfoListModel> activityTeamInfoListModels = activityTeamInfoList.stream()
                .map(activityTeamInfo -> ActivityTeamInfoListModel.infoBuild(activityTeamInfo,
                        (activityTeamInfoListModelBuilder) -> {
                            TeamProductDiscountPriceModel discountPrice = activityTeamProductSkuInfoService.getDiscountPrice(activityTeamInfo.getId());
                            TeamProductStockModel teamProductStockModel = activityTeamProductSkuInfoService.getStock(activityTeamInfo.getId());
                            Long teamOpenTypeNumber = activityTeamOpenInfoService.getTeamOpenTypeNumberByTeamId(activityTeamInfo.getId());
                            Long teamInTypeNumber = activityTeamOpenInfoService.getTeamInTypeNumberByTeamId(activityTeamInfo.getId());

                            return activityTeamInfoListModelBuilder
                                    .openTeamNumber(teamOpenTypeNumber)
                                    .addTeamNumber(teamInTypeNumber)
                                    .attendNumber(teamOpenTypeNumber + teamInTypeNumber)
                                    .teamPrice(discountPrice.getTeamPrice())
                                    .productPrice(discountPrice.getProductPrice())
                                    .teamStock(teamProductStockModel.getTeamStock())
                                    .remainStock(teamProductStockModel.getRemainStock())
                                    .build();
                        })).toList();
        TableDataInfo<ActivityTeamInfoListModel> tableDataInfo = new TableDataInfo<>();
        tableDataInfo.setTotal(activityTeamInfoPage.getTotal());
        tableDataInfo.setRows(activityTeamInfoListModels);
        return tableDataInfo;
    }

    @Override
    public boolean saveActivityTeamInfo(ActivityTeamInfoAddModel activityTeamInfoAddModel) {
        ActivityTeamInfo activityTeamInfo = ActivityTeamInfo.TeamSaveBuild(ActivityTeamInfoSaveModel.activityTeamInfoSaveModelBuild(activityTeamInfoAddModel));
        boolean save = this.save(activityTeamInfo);
        Assert.isTrue(save,"添加失败");
        activityTeamInfoAddModel.getActivityTeamProductSkuAddModelList().forEach(activityTeamProductSkuAddModel -> activityTeamProductSkuAddModel.setTeamId(activityTeamInfo.getId()));
        activityTeamProductSkuInfoService.batchSave(activityTeamInfoAddModel.getActivityTeamProductSkuAddModelList());
        return save;
    }

    @Override
    public TeamInfoFindByIdRespModel findTeamById(Long id) {
        ActivityTeamInfo byId = this.getById(id);
        List<ActivityTeamProductSkuModel> skuByTeamId = activityTeamProductSkuInfoService.findSkuByTeamId(id);
        return TeamInfoFindByIdRespModel.findSkuSumList(byId, (teamInfoFindByIdRespModelBuilder) -> {
                return teamInfoFindByIdRespModelBuilder.projectSkuInfoAddReqList(skuByTeamId)
                        .build();
        });
    }

    @Override
    public boolean updateByTeamId(ActivityTeamInfoUpdReqModel activityTeamInfoUpdReqModel) {
        boolean update = this.updateById(ActivityTeamInfo.TeamUpdateBuild(activityTeamInfoUpdReqModel));
        Assert.isTrue(update,"修改失败");
        activityTeamProductSkuInfoService.updateBath(activityTeamInfoUpdReqModel.getActivityTeamProductSkuModelList());
        return update;
    }
}
